@@ -53,15 +53,78 @@ extern "C" {
 
 extern int IS_TURN_SERVER;
 
+/* ALPN */
+
+#define OPENSSL_FIRST_ALPN_VERSION (0x10002003L)
+
+#define STUN_ALPN "stun.nat-discovery"
+#define TURN_ALPN "stun.turn"
+#define HTTP_ALPN "http/1.1"
+
+#if OPENSSL_VERSION_NUMBER >= OPENSSL_FIRST_ALPN_VERSION
+#define ALPN_SUPPORTED 1
+#else
+#define ALPN_SUPPORTED 0
+#endif
+
+/* TLS */
+
+#if defined(TURN_NO_TLS)
+
+	#define TLS_SUPPORTED 0
+	#define TLSv1_1_SUPPORTED 0
+	#define TLSv1_2_SUPPORTED 0
+
+#else
+
+	#define TLS_SUPPORTED 1
+
+	#if defined(SSL_OP_NO_TLSv1_1)
+		#define TLSv1_1_SUPPORTED 1
+	#else
+		#define TLSv1_1_SUPPORTED 0
+	#endif
+
+	#if defined(SSL_OP_NO_TLSv1_2)
+		#define TLSv1_2_SUPPORTED 1
+	#else
+		#define TLSv1_2_SUPPORTED 0
+	#endif
+
+#endif
+
+#if defined(TURN_NO_DTLS) || !defined(DTLS_CTRL_LISTEN)
+
+	#define DTLS_SUPPORTED 0
+	#define DTLSv1_2_SUPPORTED 0
+
+#else
+
+	#define DTLS_SUPPORTED 1
+
+#if defined(SSL_OP_NO_DTLSv1_2)
+		#define DTLSv1_2_SUPPORTED 1
+	#else
+		#define DTLSv1_2_SUPPORTED 0
+	#endif
+
+#endif
+
+#if OPENSSL_VERSION_NUMBER >= OPENSSL_FIRST_ALPN_VERSION
+#define SSL_SESSION_ECDH_AUTO_SUPPORTED 1
+#else
+#define SSL_SESSION_ECDH_AUTO_SUPPORTED 0
+#endif
+
 /////////// SSL //////////////////////////
 
 enum _TURN_TLS_TYPE {
 	TURN_TLS_NO=0,
 	TURN_TLS_SSL23,
 	TURN_TLS_v1_0,
-#if defined(SSL_TXT_TLSV1_1)
+#if TLSv1_1_SUPPORTED
 	TURN_TLS_v1_1,
-#if defined(SSL_TXT_TLSV1_2)
+#if TLSv1_2_SUPPORTED
 	TURN_TLS_v1_2,
 #endif
 #endif
