@@ -175,4 +175,9 @@ struct _encoded_oauth_token {
 
 typedef struct _encoded_oauth_token encoded_oauth_token;
 
+////////////// SSODA ///////////////////
+
+#define STUN_ATTRIBUTE_ADDITIONAL_ADDRESS_FAMILY (0x8032)
+#define STUN_ATTRIBUTE_ADDRESS_ERROR_CODE (0x8033)
+
 #endif //__LIB_TURN_MSG_DEFS_NEW__
